#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2021-1-22
@desc: 通过函数调用uuid-service服务
"""
from yzcore.default_settings import default_setting as settings
from yzcore.request import request

__all__ = [
    "generate_uuid",
    "explain_uuid",
    "translate2timestamp",
    "make_uuid",
]


if settings.ID_URL is None:
    raise EnvironmentError("Error: 'ID_URL' is None.")


def generate_uuid(limit: int = 1) -> int or list:
    """
    根据参数生成一定数量的ID

    :param limit:
    :return:
            1: 6918460366603157505
            2: [6918460407405346817, 6918460407405347841,...]
    """
    url = f"{settings.ID_URL}{settings.GENERATE_UUID_PATH}"
    result, status_code = request(
        'get', url, params={'limit': limit}
    )
    if status_code == 200:
        if limit == 1:
            return int(result['id'])
        else:
            return [int(_id) for _id in result['id_list']]


def explain_uuid(long_id: int) -> dict:
    """
    解析长整型ID为 id_object

    :param long_id: 6918460366603157505
    :return: {
              "machine_id": 1,
              "sequence": 0,
              "time_duration": 867377,
              "generate_method": 2,
              "mode_type": 1,
              "version": 0
            }
    """
    url = f"{settings.ID_URL}{settings.EXPLAIN_UUID_PATH}"
    result, status_code = request(
        'get', url, params={'long_id': long_id}
    )
    if status_code == 200:
        return result


def translate2timestamp(time_duration: int):
    """
    把时间间隔转换为时间戳

    :param time_duration: 时间间隔
    :return: {
               "timestamp": 1611219039,
               "datetime": "2021-01-21T16:50:39"
             }
    """
    url = f"{settings.ID_URL}{settings.TRANSLATE_PATH}"
    result, status_code = request(
        'get', url, params={'time_duration': time_duration})
    if status_code == 200:
        return result


def make_uuid(sequence: int, timestamp: int,
            machine: int=None, method: int=None,
            mtype: int=None, version: int=None
):
    """
    根据传入参数合成长整型id

    :param sequence:
    :param timestamp:
    :param machine:
    :param method:
    :param mtype:
    :param version:
    :return:
    """
    data = {
        "sequence": sequence,
        "timestamp": timestamp,
        "machine": machine,
        "method": method,
        "mtype": mtype,
        "version": version
    }
    data = {k: v for k, v in data.items() if v is not None}
    url = f"{settings.ID_URL}{settings.MAKE_UUID_PATH}"
    result, status_code = request('post', url, json=data)
    if status_code == 200:
        return int(result['id'])





try:
    import obs

    ACL_TYPE = {
        "private": obs.HeadPermission.PRIVATE,
        "onlyread": obs.HeadPermission.PUBLIC_READ,
        "readwrite": obs.HeadPermission.PUBLIC_READ_WRITE,
        "bucket_read": obs.HeadPermission.PUBLIC_READ_DELIVERED,  # 桶公共读，桶内对象公共读。
        "bucket_readwrite": obs.HeadPermission.PUBLIC_READ_WRITE_DELIVERED,  # 桶公共读写，桶内对象公共读写。
        "owner_full_control": obs.HeadPermission.BUCKET_OWNER_FULL_CONTROL,  # 桶或对象所有者拥有完全控制权限。
    }
    # 存储类型
    STORAGE_CLS = {
        "standard": obs.StorageClass.STANDARD,  # 标准类型
        "ia": obs.StorageClass.WARM,  # 低频访问类型
        # "archive": oss2.BUCKET_STORAGE_CLASS_ARCHIVE,  # 归档类型
        "cold_archive": obs.StorageClass.COLD,  # 冷归档类型
    }

    # 冗余类型
    # REDUNDANCY_TYPE = {
    #     "lrs": oss2.BUCKET_DATA_REDUNDANCY_TYPE_LRS,  # 本地冗余
    #     "zrs": oss2.BUCKET_DATA_REDUNDANCY_TYPE_ZRS,  # 同城冗余（跨机房）
    # }

except ImportError:
    pass



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@author: zhouwei
@date: 2023/04/17
@desc: azure blob对象存储封装
"""

from datetime import datetime, timedelta
import os
from io import BufferedReader, BytesIO
from typing import Union
from urllib.parse import unquote

from yzcore.extensions.storage.base import StorageManagerBase, StorageRequestError, IMAGE_FORMAT_SET
from yzcore.extensions.storage.schemas import AzureConfig
from yzcore.utils.time_utils import datetime2str


try:
    from azure.storage.blob import BlobServiceClient, ContentSettings, ContainerClient, generate_blob_sas,\
        BlobSasPermissions
    from azure.core.exceptions import ResourceExistsError
except:
    BlobServiceClient = None


class AzureManager(StorageManagerBase):

    def __init__(self, conf: AzureConfig):
        super(AzureManager, self).__init__(conf)
        self.connection_string = conf.connection_string
        self.account_key = conf.account_key
        self.account_name = conf.account_name

        self.__init()

    def __init(self, bucket_name=None):
        """初始化对象"""

        if BlobServiceClient is None:
            raise ImportError("'azure-storage-blob' must be installed to use AzureManager")

        if bucket_name:
            self.bucket_name = bucket_name

        self.blob_service_client = BlobServiceClient.from_connection_string(self.connection_string)
        self.container_client = self.blob_service_client.get_container_client(self.bucket_name)

        if self.cache_path:
            try:
                os.makedirs(self.cache_path)
            except OSError:
                pass

    def create_bucket(self, bucket_name):
        try:
            self.blob_service_client.create_container(bucket_name)
        except ResourceExistsError:
            pass
        self.bucket_name = bucket_name

    def get_bucket_cors(self):
        cors_dict = {
            'allowed_origins': [],
            'allowed_methods': [],
            'allowed_headers': [],
        }
        for cors_rule in self.blob_service_client.get_service_properties()['cors']:
            cors_dict['allowed_origins'] = cors_rule.allowed_origins
            cors_dict['allowed_methods'] = cors_rule.allowed_methods
            cors_dict['allowed_headers'] = cors_rule.allowed_headers
        return cors_dict

    def list_buckets(self):
        return self.blob_service_client.list_containers()

    def is_exist_bucket(self, bucket_name=None):
        if bucket_name is None:
            bucket_name = self.bucket_name
        container_client = self.blob_service_client.get_container_client(bucket_name)
        return container_client.exists()

    def delete_bucket(self, bucket_name=None):
        if bucket_name is None:
            bucket_name = self.bucket_name
        self.blob_service_client.delete_container(bucket_name)

    def get_sign_url(self, key, expire=0):
        expire_time = datetime.utcnow() + timedelta(seconds=expire or self.private_expire_time)
        blob_client = self.container_client.get_blob_client(blob=key)
        sas_sign = generate_blob_sas(
            account_name=self.account_name, container_name=self.bucket_name, blob_name=key, account_key=self.account_key,
            expiry=expire_time, permission=BlobSasPermissions(read=True)
        )
        return f'{blob_client.url}?{sas_sign}'

    def post_sign_url(self, key):
        pass

    def put_sign_url(self, key):
        pass

    def get_file_url(self, key, with_scheme=False):
        if not any((self.image_domain, self.asset_domain)):
            resource_url = u"//{}/{}/{}".format(self.endpoint, self.bucket_name, key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.image_domain, bucket=self.bucket_name, key=key)
        else:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.asset_domain, bucket=self.bucket_name, key=key)
        if with_scheme:
            resource_url = self.scheme + ':' + resource_url
        return resource_url

    @property
    def host(self):
        return u'//{}/{}'.format(self.endpoint, self.bucket_name)

    def get_key_from_url(self, url, urldecode=False):
        """从URL中获取对象存储key"""
        path = url.split(self.bucket_name + '/')[-1]
        if urldecode:
            path = unquote(path)
        return path

    def iter_objects(self, prefix='', marker=None, delimiter=None, max_keys=100):
        objects = self.container_client.list_blobs(name_starts_with=prefix, results_per_page=max_keys)
        _result = []
        for obj in objects:
            _result.append({
                'key': obj.name,
                'url': self.get_file_url(obj.name),
                'size': obj.size,
            })
        return _result

    def get_object_meta(self, key: str):
        blob_client = self.container_client.get_blob_client(blob=key)
        metadata = blob_client.get_blob_properties()
        return {
            'etag': '',  # metadata['etag'].strip('"').lower(), azure的etag计算方式和oss/obs/minio 不一样
            'size': metadata['size'],
            'last_modified': datetime2str(metadata['last_modified']),
            'content_type': metadata['content_settings']['content_type']
        }

    def update_file_headers(self, key, headers: dict):
        blob_client = self.container_client.get_blob_client(blob=key)
        blob_client.set_http_headers(ContentSettings(**headers))
        return True

    def file_exists(self, key):
        blob_client = self.container_client.get_blob_client(blob=key)
        return blob_client.exists()

    def download_stream(self, key, **kwargs):
        blob_client = self.container_client.get_blob_client(blob=key)
        stream = BytesIO()
        blob_client.download_blob().readinto(stream)
        stream.seek(0)
        return stream

    def download_file(self, key, local_name, **kwargs):
        blob_client = self.container_client.get_blob_client(blob=key)
        with open(local_name, 'wb') as f:
            f.write(blob_client.download_blob().readall())

    def upload(self, filepath: Union[str, BufferedReader], key: str):
        try:
            blob_client = self.container_client.get_blob_client(blob=key)
            if isinstance(filepath, str):
                with open(filepath, 'rb') as f:
                    blob_client.upload_blob(f)
            else:
                blob_client.upload_blob(filepath)
            return self.get_file_url(key)
        except Exception as e:
            raise StorageRequestError(f'azure blob upload error: {e}')

    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict = None,
            callback_content_type: str = "application/json",
            callback_directly: bool = True,
    ):
        """
        授权给第三方上传
        :param filepath:
        :param callback_url: 对象存储的回调地址
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"
        :param callback_directly: True 由对象存储自动发起回调 / False 需要前端主动发起回调
        :return:
        """
        pass




import os
import shutil
import stat
from os import path

import yzcore
from . import CommandError
from yzrpc.commands import CommandBase


class TemplateCommand(CommandBase):

    requires_system_checks = False
    # The supported URL schemes
    url_schemes = ['http', 'https', 'ftp']
    # Rewrite the following suffixes when determining the target filename.
    rewrite_template_suffixes = (
        # Allow shipping invalid .py files without byte-compilation.
        ('.py-tpl', '.py'),
    )

    def add_arguments(self, parser):
        parser.add_argument('name', help='Name of the application or project.')
        parser.add_argument('directory', nargs='?', help='Optional destination directory')
        # parser.add_argument('--template', help='The path or URL to load the template from.')

    def handle(self, app_or_project, name, target=None, **options):
        """

        :param app_or_project: "app" or "project"
        :param name: 工程或者应用名称
        :param target:
        :param options:
        :return:
        """
        self.app_or_project = app_or_project
        # self.paths_to_remove = []
        # self.verbosity = options['verbosity']

        self.validate_name(name, app_or_project)

        # if some directory is given, make sure it's nicely expanded
        if target is None:
            top_dir = path.join(os.getcwd(), name)
            try:
                os.makedirs(top_dir)
            except FileExistsError:
                raise CommandError("'%s' already exists" % top_dir)
            except OSError as e:
                raise CommandError(e)
        else:
            top_dir = os.path.abspath(path.expanduser(target))
            if not os.path.exists(top_dir):
                raise CommandError("Destination directory '%s' does not "
                                   "exist, please create it first." % top_dir)
            if self.app_or_project == "app":
                top_dir = path.join(top_dir, name)
                try:
                    os.makedirs(top_dir)
                except FileExistsError:
                    raise CommandError("'%s' already exists" % top_dir)
                except OSError as e:
                    raise CommandError(e)

        base_name = '%s_name' % app_or_project
        base_subdir = '%s_template' % app_or_project

        template_dir = path.join(yzcore.__path__[0], 'templates', base_subdir)
        prefix_length = len(template_dir) + 1

        for root, dirs, files in os.walk(template_dir):

            path_rest = root[prefix_length:]
            relative_dir = path_rest.replace(base_name, name)
            # relative_dir = path_rest.replace(base_subdir, name)
            if relative_dir:
                target_dir = path.join(top_dir, relative_dir)
                if not path.exists(target_dir):
                    os.mkdir(target_dir)

            for dirname in dirs[:]:
                if dirname.startswith('.') or dirname == '__pycache__':
                    dirs.remove(dirname)

            for filename in files:
                if filename.endswith(('.pyo', '.pyc', '.py.class')):
                    # Ignore some files as they cause various breakages.
                    continue
                old_path = path.join(root, filename)
                new_path = path.join(top_dir, relative_dir,
                                     filename.replace(base_name, name))
                for old_suffix, new_suffix in self.rewrite_template_suffixes:
                    if new_path.endswith(old_suffix):
                        new_path = new_path[:-len(old_suffix)] + new_suffix
                        break  # Only rewrite once

                if path.exists(new_path):
                    raise CommandError("%s already exists, overlaying a "
                                       "project or app into an existing "
                                       "directory won't replace conflicting "
                                       "files" % new_path)

                shutil.copyfile(old_path, new_path)

                # if self.verbosity >= 2:
                self.stdout.write("Creating %s\n" % new_path)
                try:
                    shutil.copymode(old_path, new_path)
                    self.make_writeable(new_path)
                except OSError:
                    self.stderr.write(
                        "Notice: Couldn't set permission bits on %s. You're "
                        "probably using an uncommon filesystem setup. No "
                        "problem." % new_path, self.style.NOTICE)

    def validate_name(self, name, app_or_project):
        a_or_an = 'an' if app_or_project == 'app' else 'a'
        if name is None:
            raise CommandError('you must provide {an} {app} name'.format(
                an=a_or_an,
                app=app_or_project,
            ))

    def make_writeable(self, filename):
        """
        Make sure that the file is writeable.
        Useful if our source is read-only.
        """
        if not os.access(filename, os.W_OK):
            st = os.stat(filename)
            new_permissions = stat.S_IMODE(st.st_mode) | stat.S_IWUSR
            os.chmod(filename, new_permissions)





# copied from django.utils.functional.cached_property
class cached_property:
    """
    Decorator that converts a method with a single self argument into a
    property cached on the instance.

    A cached property can be made out of an existing method:
    (e.g. ``url = cached_property(get_absolute_url)``).
    """

    name = None

    @staticmethod
    def func(instance):
        raise TypeError(
            "Cannot use cached_property instance without calling "
            "__set_name__() on it."
        )

    def __init__(self, func):
        self.real_func = func
        self.__doc__ = getattr(func, "__doc__")

    def __set_name__(self, owner, name):
        if self.name is None:
            self.name = name
            self.func = self.real_func
        elif name != self.name:
            raise TypeError(
                "Cannot assign the same cached_property to two different names "
                "(%r and %r)." % (self.name, name)
            )

    def __get__(self, instance, cls=None):
        """
        Call the function and put the return value in instance.__dict__ so that
        subsequent attribute access on the instance returns the cached value
        instead of calling cached_property.__get__().
        """
        if instance is None:
            return self
        res = instance.__dict__[self.name] = self.func(instance)
        return res


# copied from django.utils.functional.classproperty
class classproperty:
    """
    Decorator that converts a method with a single cls argument into a property
    that can be accessed directly from the class.
    """

    def __init__(self, method=None):
        self.fget = method

    def __get__(self, instance, cls=None):
        return self.fget(cls)

    def getter(self, method):
        self.fget = method
        return self



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@author: zhouwei
@date: 2022/11/09
@desc: minio对象存储封装
"""
import json
import os
from urllib.parse import unquote

from yzcore.extensions.storage.base import StorageManagerBase, StorageRequestError, logger, IMAGE_FORMAT_SET
from yzcore.extensions.storage.schemas import MinioConfig
from yzcore.utils.time_utils import datetime2str
from datetime import timedelta, datetime


try:
    from minio import Minio
    from minio.datatypes import PostPolicy
    from minio.commonconfig import CopySource
    from minio.error import S3Error
except:
    Minio = None


class MinioManager(StorageManagerBase):

    def __init__(self, conf: MinioConfig):
        super(MinioManager, self).__init__(conf)
        self.internal_endpoint = conf.internal_endpoint
        self.internal_minioClient = None

        self.__init()

    def __init(self, bucket_name=None):
        """初始化对象"""

        if Minio is None:
            raise ImportError("'minio' must be installed to use MinioManager")

        if bucket_name:
            self.bucket_name = bucket_name

        self.minioClient = Minio(
            self.endpoint,
            access_key=self.access_key_id,
            secret_key=self.access_key_secret,
            secure=True if self.scheme == 'https' else False,
        )

        if self.internal_endpoint:
            self.internal_minioClient = Minio(
                self.internal_endpoint,
                access_key=self.access_key_id,
                secret_key=self.access_key_secret,
                secure=False,
            )

        if self.cache_path:
            try:
                os.makedirs(self.cache_path)
            except OSError:
                pass

    def _internal_minio_client_first(self):
        """优先使用内网连接minio服务"""
        if self.internal_endpoint:
            client = self.internal_minioClient
        else:
            client = self.minioClient
        return client

    def get_bucket_cors(self):
        """返回的内容格式和OSS/OBS差异太大"""
        client = self._internal_minio_client_first()
        result = client.get_bucket_policy(self.bucket_name)
        return json.loads(result)

    def set_bucket_cors(self, policy: dict):
        client = self._internal_minio_client_first()
        return client.set_bucket_policy(self.bucket_name, policy)

    def _cors_check(self):
        passed = False
        action_slots = ['s3:AbortMultipartUpload', 's3:DeleteObject', 's3:GetObject',
                        's3:ListMultipartUploadParts', 's3:PutObject']

        cors_dict = self.get_bucket_cors()
        logger.debug(f'_cors_check: {cors_dict}')
        for cors in cors_dict['Statement']:
            effect = cors['Effect']
            resource = cors['Resource'][0]
            actions = cors['Action']
            if effect == 'Allow':
                if resource == f'arn:aws:s3:::{self.bucket_name}/*' or resource == 'arn:aws:s3:::*':
                    if all([False for i in action_slots if i not in actions]):
                        passed = True
        return passed

    def create_bucket(self, bucket_name=None):
        """创建bucket，并且作为当前操作bucket"""
        client = self._internal_minio_client_first()
        client.make_bucket(bucket_name)
        self.bucket_name = bucket_name

    def list_buckets(self):
        client = self._internal_minio_client_first()
        return client.list_buckets()

    def is_exist_bucket(self, bucket_name=None):
        client = self._internal_minio_client_first()
        if bucket_name is None:
            bucket_name = self.bucket_name
        return client.bucket_exists(bucket_name)

    def delete_bucket(self, bucket_name=None):
        client = self._internal_minio_client_first()
        if bucket_name is None:
            bucket_name = self.bucket_name
        return client.remove_bucket(bucket_name)

    def get_sign_url(self, key, expire=0):
        expire_time = timedelta(seconds=expire or self.private_expire_time)
        url = self.minioClient.presigned_get_object(self.bucket_name, key, expires=expire_time)
        return '//' + url.split('//', 1)[-1]

    def post_sign_url(self, key):
        expire_time = datetime.now() + timedelta(seconds=self.policy_expire_time)
        policy = PostPolicy(bucket_name=self.bucket_name, expiration=expire_time)
        policy.add_starts_with_condition('$key', key)
        # policy.add_content_length_range_condition(1, 1024*1024*1024)  # 限制文件大小
        return self.minioClient.presigned_post_policy(policy)

    def put_sign_url(self, key):
        return self.minioClient.presigned_put_object(self.bucket_name, key)

    def iter_objects(self, prefix='', marker=None, delimiter=None, max_keys=100):
        client = self._internal_minio_client_first()
        objects = client.list_objects(self.bucket_name, prefix=prefix, recursive=True)
        _result = []
        for obj in objects:
            _result.append({
                'key': obj.object_name,
                'url': self.get_file_url(obj.object_name),
                'size': obj.size,
            })
        return _result

    def get_object_meta(self, key: str):
        """获取文件基本元信息，包括该Object的ETag、Size（文件大小）、LastModified，Content-Type，并不返回其内容"""
        client = self._internal_minio_client_first()
        meta = client.stat_object(self.bucket_name, key)
        return {
            'etag': meta.etag,
            'size': meta.size,
            'last_modified': datetime2str(meta.last_modified),
            'content_type': meta.content_type,
        }

    def update_file_headers(self, key, headers: dict):
        client = self._internal_minio_client_first()
        client.copy_object(self.bucket_name, key, CopySource(self.bucket_name, key), metadata=headers, metadata_directive='REPLACE')
        return True

    def file_exists(self, key):
        client = self._internal_minio_client_first()
        try:
            client.stat_object(self.bucket_name, key)
            return True
        except S3Error as e:
            if e.code == 'NoSuchKey':
                return False
            raise e

    def download_stream(self, key, **kwargs):
        client = self._internal_minio_client_first()
        return client.get_object(self.bucket_name, key)

    def download_file(self, key, local_name, **kwargs):
        client = self._internal_minio_client_first()
        client.fget_object(self.bucket_name, key, local_name)

    def upload(self, filepath, key: str):
        """
        文件上传
        :param filepath:
        :param key:
        """
        client = self._internal_minio_client_first()
        try:
            content_type = self.parse_content_type(key)

            if isinstance(filepath, str):
                client.fput_object(self.bucket_name, key, filepath, content_type=content_type)
            else:
                client.put_object(self.bucket_name, key, filepath, length=-1, content_type=content_type, part_size=1024*1024*5)
            return self.get_file_url(key)
        except Exception as e:
            raise StorageRequestError(f'minio upload error: {e}')

    def get_policy(
            self,
            filepath: str,
            callback_url: str,
            callback_data: dict = None,
            callback_content_type: str = "application/json",
            callback_directly: bool = False,
    ):
        """
        授权给第三方上传
        :param filepath:
        :param callback_url: 对象存储的回调地址
        :param callback_data: 需要回传的参数
        :param callback_content_type: 回调时的Content-Type
               "application/json"
               "application/x-www-form-urlencoded"
        :param callback_directly:  False 需要前端主动发起回调 minio没有回调功能，只能由前端发起
        :return:
        """
        form_data = self.post_sign_url(filepath)
        data = {
            'mode': 'minio',
            'host': f'{self.scheme}:{self.host}',
            'dir': filepath,
            'success_action_status': 200,
            'callback': {'url': callback_url, 'data': callback_data},
            # 'Content-Type': '上传时指定Content-Type',
            **form_data,
        }
        return data

    @property
    def host(self):
        return u'//{}/{}'.format(self.endpoint, self.bucket_name)

    def get_key_from_url(self, url, urldecode=False):
        """从URL中获取对象存储key"""
        path = url.split(self.bucket_name + '/')[-1]
        if urldecode:
            path = unquote(path)
        return path

    def get_file_url(self, key, with_scheme=False):
        if not any((self.image_domain, self.asset_domain)):
            resource_url = u"//{}/{}/{}".format(self.endpoint, self.bucket_name, key)
        elif key.split('.')[-1].lower() in IMAGE_FORMAT_SET:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.image_domain, bucket=self.bucket_name, key=key)
        else:
            resource_url = u"//{domain}/{bucket}/{key}".format(
                domain=self.asset_domain, bucket=self.bucket_name, key=key)
        if with_scheme:
            resource_url = self.scheme + ':' + resource_url
        return resource_url



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-
@desc: ...
"""
from .crypto import get_random_string


def get_random_secret_key():
    """
    Return a 50 character random string usable as a SECRET_KEY setting value.
    """
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    return get_random_string(50, chars)


from .time_utils import *



from typing import Optional
from pydantic import BaseModel, root_validator
from .const import StorageMode, Scheme


class BaseConfig(BaseModel):
    mode: StorageMode
    access_key_id: str
    access_key_secret: str
    bucket_name: str
    endpoint: str
    scheme: Optional[Scheme] = Scheme.https
    image_domain: Optional[str] = None
    asset_domain: Optional[str] = None

    cache_path: Optional[str] = '.'
    policy_expire_time: Optional[int]  # 上传签名有效时间
    private_expire_time: Optional[int]  # 私有桶访问链接有效时间

    @root_validator
    def base_validator(cls, values):
        values['mode'] = values['mode'].value
        if not values['policy_expire_time']:
            values['policy_expire_time'] = 30
        if not values['private_expire_time']:
            values['private_expire_time'] = 3600
        return values


class OssConfig(BaseConfig):
    internal_endpoint: str = None


class ObsConfig(BaseConfig):
    pass


class MinioConfig(BaseConfig):
    internal_endpoint: str = None


class AzureConfig(BaseConfig):
    connection_string: str
    account_key: str
    account_name: str
    access_key_id: Optional[str] = None
    access_key_secret: Optional[str] = None



#!/usr/bin/python3.7+
# -*- coding:utf-8 -*-
"""
@auth: xxx
@date: 2020-9-
@desc: ...
"""
from sqlalchemy import Table, Column, ForeignKey
from sqlalchemy import String, Integer, DateTime, Float
# from sqlalchemy.orm import relationship
# from datetime import datetime

from yzcore.db.sqlalchemy_crud_base import Base


class Permissions(Base):
    __tablename__ = 'permissions'
    id = Column(Integer, primary_key=True)


#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-6-22
@desc: 非关系数据库的增删改查封装
"""
from typing import Any, Dict, Generic, List, Optional, Type, TypeVar, Union

from pydantic import BaseModel, AnyUrl
from pymongo.client_session import ClientSession

try:
    from pymongo import InsertOne, DeleteOne, ReplaceOne, UpdateMany
    from pymongo.collection import Collection
    from pymongo import MongoClient
except:
    pass

ModelType = TypeVar("ModelType", bound=str)
CreateSchemaType = TypeVar("CreateSchemaType", bound=BaseModel)
UpdateSchemaType = TypeVar("UpdateSchemaType", bound=BaseModel)
DictorList = TypeVar("DictorList", dict, list)


# DictorList = Union[Dict, List]


class MongoCRUDBase(Generic[CreateSchemaType, UpdateSchemaType]):
    def __init__(
            self,
            collection_name: ModelType,
            db_name: str = "test_db",
            db_url: AnyUrl = "mongodb://localhost:27017/",
            client: ClientSession = None
    ):
        if client:
            self.client = client
        else:
            self.client = MongoClient(db_url)
        self.db = self.client[db_name]
        self.collection = self.db[collection_name]
        # self.coll_name = collection_name

    def count(self, opt: dict = None, session: ClientSession = None):
        """
        统计数目

        :param opt:
        :param session: 事务操作
        :return:
        """
        if opt:
            return self.collection.count_documents(opt, session=session)
        return self.collection.estimated_document_count(session=session)

    def get(self, opt: dict = None, is_logical_del: bool = False,
            select_col: DictorList = None, session: ClientSession = None):
        """
        查询操作

        :param opt:
        :param is_logical_del: 是否逻辑删除
        :param select_col: 应在结果集中返回的字段名列表，或指定要包含或排除的字段的dict
        :param session: 事务操作
        :return:
        """
        if is_logical_del:
            opt.update({"isDelete": False})
        return self.collection.find_one(opt, projection=select_col,
                                        session=session)

    def list(self, opt: dict = None, select_col: DictorList = None,
             limit: int = 0, offset: int = 0, sort: List[tuple] = None,
             is_logical_del: bool = False, session: ClientSession = None
             ):
        """
        `projection`（可选）：应在结果集中返回的字段名列表，或指定要包含或排除的字段的dict。
        如果“projection”是一个列表，则始终返回“_id”。使用dict从结果中排除字段
        （例如projection={'_id'：False}）。

        :param opt:
        :param select_col: {'_id': 0, 'author': 1, 'age': 1}
        :param limit: 0
        :param offset: 0
        :param sort: [
                        ('field1', pymongo.ASCENDING),
                        ('field2', pymongo.DESCENDING)
                    ]
        :param is_logical_del:
        :param session: 事务操作
        :return:
        """
        if opt is None:
            opt = dict()
        if is_logical_del:
            opt.update({"isDelete": False})
        data = dict(
            filter=opt,
            projection=select_col,
            skip=offset,
            limit=limit,
            sort=sort
        )
        results = list(self.collection.find(**data, session=session))
        return results

    def create(self, data: DictorList, is_return_obj: bool = False,
               session: ClientSession = None):
        """
        插入操作

        :param data:
        :param is_return_obj:
        :param session: 事务操作
        :return:
        """
        if isinstance(data, dict):
            result = self.collection.insert_one(data, session=session)
            if is_return_obj:
                result = self.collection.find_one({'_id': result.inserted_id},
                                                  session=session)
        elif isinstance(data, list):
            result = self.collection.insert_many(data, session=session)
            if is_return_obj:
                result = self.list({'_id': {'$in': result.inserted_ids}},
                                   session=session)
        else:
            raise Exception('Create failed!')
        return result

    def update(self, opt, data: Dict, is_many: bool = False,
               is_set: bool = True, session: ClientSession = None):
        """
        更新操作

        :param opt: 查询条件
                    opt={'field1': 'xxx'}
                    opt={'field1': 'xxx', 'field2': 123}
                    opt={'field1': {'$gt': 'a'}, 'field2': {'$regex': '^d'}}
        :param data: 需要更新的数据：
                    {'field': 'xxx'}
        :param is_many: 是否批量更新，默认为False
        :param is_set: 是否设置$set，默认为True
        :param session: 事务操作
        :return:
        """
        if is_set:
            update = {"$set": data}
        else:
            update = data
        if not is_many:
            result = self.collection.update_one(opt, update, session=session)
            # result = self.collection.find_one_and_update(opt, update)
        else:
            result = self.collection.update_many(opt, update, session=session)

        if result.acknowledged:
            return result

    def delete(self, opt, is_logical_del: bool = False, is_many: bool = False,
               session: ClientSession = None):
        """
        删除操作: 默认执行逻辑删除，当physical为True时，执行物理删除

        :param opt: 搜索条件
        :param is_logical_del: 是否逻辑删除
        :param is_many: 是否删除多个
        :param session: 事务操作
        :return:
        """
        if is_logical_del:
            update = {"$set": {"isDelete": True}}
            if not is_many:
                result = self.collection.update_one(filter=opt, update=update,
                                                    session=session)
            else:
                result = self.collection.update_many(filter=opt, update=update,
                                                     session=session)
            return result.modified_count
        else:
            if not is_many:
                result = self.collection.delete_one(filter=opt, session=session)
            else:
                result = self.collection.delete_many(filter=opt,
                                                     session=session)
            return result.deleted_count

    def batch_update(self, bulk_update_datas: List[dict], session: ClientSession = None):
        """
        批量更新

        :param bulk_update_datas: 格式:[{"opt": {}, "data": {}}]
        :param session: 事务操作
        :return:
        """
        if not bulk_update_datas:
            return 0
        requests = []
        for bulk_update_data in bulk_update_datas:
            requests.append(UpdateMany(bulk_update_data['opt'], bulk_update_data['data']))
        result = self.collection.bulk_write(requests=requests, session=session)
        return result.modified_count

    def aggregate(self, pipeline: List[dict], session: ClientSession = None, **kwargs):
        """
        聚合管道
        :param pipeline:
        :param session: 事务操作
        :return:
        """
        cursor = self.collection.aggregate(pipeline, session=session, **kwargs)
        return list(cursor)


if __name__ == '__main__':
    db = MongoCRUDBase('hello_cml')
    print(db.count())



import functools
from yzcore.extensions.storage.base import StorageRequestError


def wrap_request_return_bool(func):
    """"""

    @functools.wraps(func)
    def wrap_func(*args, **kwargs):
        try:
            resp = func(*args, **kwargs)
            if resp.status < 300:
                return True
            if resp.status == 403:
                raise StorageRequestError(
                    f"static_code: {resp.status}, errorCode: {resp.errorCode}. Message: {resp.errorMessage}.")
            else:
                return False
        except:
            import traceback
            print(traceback.format_exc())

    return wrap_func



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-12-2
@desc: ...
"""
from enum import Enum


class TaskType(str, Enum):
    MODEL = 'model'
    TEXTURE = 'texture'
    CUBEMAP = 'cubemap'
    SCENE = 'scene'
    UI = 'ui'
    FILE = 'file'


class TaskAction(str, Enum):
    CONVERT = 'convert'
    PROCESS = 'process'
    PACKAGE = 'package'
    DOWNLOAD = 'download'
    PUBLISH = 'publish'



#!/usr/bin/python3.6.8+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-12-30
@desc: 时间方面的处理函数
"""
import time
import datetime

# 北京时区
Beijing = datetime.timezone(datetime.timedelta(hours=8))


__all__ = [
    "get_zero_time",
    "get_today_date",
    "datetime2timestamp",
    "timestamp2datetime",
    "datetime2str",
    "timestamp2str",
]


def get_zero_time(
        date_time: datetime = None,
        is_timestamp: bool = True
):
    """
    获取0点时间，默认获取今天的0点时间

    :param date_time:
    :param is_timestamp:
    :return:
    """
    if not date_time:
        date_time = datetime.date.today()
        if is_timestamp:
            return int(time.mktime(date_time.timetuple()))
        return date_time
    else:
        time_stamp = int(time.mktime(date_time.timetuple()))
        if is_timestamp:
            return time_stamp
        date_time = datetime.datetime.fromtimestamp(time_stamp)
    return date_time


def datetime2timestamp(date_time: datetime):
    return int(time.mktime(date_time.timetuple()))


def timestamp2datetime(time_stamp: int):
    _s = str(time_stamp)
    if len(_s) > 10 and '.' not in _s:
        _s = '.'.join((_s[:10], _s[10:]))
        time_stamp = float(_s)
    return datetime.datetime.fromtimestamp(time_stamp)


def get_today_date(is_strf: bool = False, format: str = "%Y-%m-%d"):
    """
    获取今天的日期

    :param is_strf:
    :param format:
    :return:
    """
    today = datetime.date.today()
    if is_strf:
        today = today.strftime(format)
    return today


def datetime2str(dt: datetime.datetime, fmt='%Y-%m-%d %H:%M:%S'):
    """
    datetime格式转字符串日期
    :param dt:
    :param fmt
    :return: "2023-04-18 18:54:59"
    """
    if dt.tzinfo:
        dt = dt.astimezone(Beijing)
    return dt.strftime(fmt)


def timestamp2str(timestamp, fmt='%Y-%m-%d %H:%M:%S'):
    """
    时间戳转字符串
    :param timestamp: int|float
    :param fmt:
    :return: "2023-04-18 18:54:59"
    """
    dt = datetime.datetime.fromtimestamp(timestamp, tz=Beijing)
    return dt.strftime(fmt)



#!/usr/bin/python3.6+
# -*- coding:utf-8 -*-
"""
@auth: cml
@date: 2020-9-13
@desc: ...
"""
import os
import json
import typing
from starlette.datastructures import URL
from starlette.background import BackgroundTask
from fastapi import Response as _Response
from fastapi.responses import (
    HTMLResponse, PlainTextResponse,
    JSONResponse, UJSONResponse, ORJSONResponse,
    RedirectResponse, StreamingResponse, FileResponse
)


class XMLResponse(_Response):
    media_type = "application/xml"


responses = {
    "xml": XMLResponse,
    "html": HTMLResponse,
    "plain": PlainTextResponse,
    "json": JSONResponse,
    "ujson": UJSONResponse,
    "orjson": ORJSONResponse,
    "redirect": RedirectResponse,
    "stream": StreamingResponse,
    "file": FileResponse,
}


def response(
        content: typing.Any = None,
        url: typing.Union[str, URL] = None,     # RedirectResponse 重定向使用

        path: str = None,                       # FileResponse
        filename: str = None,                   # FileResponse
        stat_result: os.stat_result = None,     # FileResponse
        method: str = None,                     # FileResponse

        status_code: int = 200,
        headers: dict = None,
        media_type: str = None,
        background: BackgroundTask = None,

        mtype: str = "orjson"
):
    if 'json' in mtype:
        content = render_data(data=content)
    elif mtype in ['plain', 'xml', 'html'] and not isinstance(content, str):
        content = json.dumps(content)

    kwargs = dict(
        content=content,
        url=url,
        path=path,
        filename=filename,
        stat_result=stat_result,
        method=method,
        status_code=status_code,
        headers=headers,
        media_type=media_type,
        background=background,
    )
    kwargs = {k: v for k, v in kwargs.items() if v is not None}
    _response_cls = responses.get(mtype)
    return _response_cls(**kwargs)


def render_data(data=None, code=10000, message='Successfully.',
                limit: int = 10, offset: int = 0, total: int = 0):
    if data is None:
        return dict(
            code=code,
            message=message,
            info=dict(),
            list=dict(
                data=[],
                pagination=False
            )
        )
    if isinstance(data, list):
        result = dict(
            code=code,
            message=message,
            info=dict(),
            list=dict(
                data=data,
                pagination=dict(
                    limit=limit,
                    offset=offset,
                    total=total
                )
            )
        )
    # elif isinstance(data, Container):
    else:
        result = dict(
            code=code,
            message=message,
            info=data,
            list=dict(
                data=[],
                pagination=False
            )
        )
    return result
